"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.hasAttribute = hasAttribute;
exports.getAttribute = getAttribute;
exports.hasLink = hasLink;
exports.getLink = getLink;
exports.hasRelationship = hasRelationship;
exports.getRelationship = getRelationship;
exports.Client = void 0;

var _url = require("url");

var _path = _interopRequireDefault(require("path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function hasAttribute(resource, attr) {
  /* eslint-disable-next-line */
  return resource.hasOwnProperty('attributes') && resource.attributes.hasOwnProperty(attr);
}

var __d__ = {};

function getAttribute(resource, attr) {
  var default_ = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : __d__;

  if (hasAttribute(resource, attr)) {
    return resource.attributes[attr];
  } else if (default_ !== __d__) {
    return default_;
  } else {
    throw new Error("Attribute field, ".concat(attr, ", is not defined on this resource."));
  }
}

function hasLink(resource, member) {
  /* eslint-disable-next-line */
  return resource.hasOwnProperty('links') && resource.links.hasOwnProperty(member);
}

function getLink(resource, member, default_) {
  if (hasLink(resource, member)) {
    return resource.links[member];
  } else if (default_) {
    return default_;
  } else {
    throw new Error("Link, ".concat(member, ", is not defined on this resource."));
  }
}

function hasRelationship(resource, rel) {
  /* eslint-disable-next-line */
  return resource.hasOwnProperty('relationships') && resource.relationships.hasOwnProperty(rel);
}

function getRelationship(resource, rel) {
  if (hasRelationship(resource, rel)) {
    return resource.relationships[rel];
  } else {
    throw new Error("Relationship field, ".concat(rel, ", is not defined on this resource."));
  }
}

var Client = /*#__PURE__*/function () {
  function Client() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Client);

    /* eslint-disable-next-line */
    if (options.hasOwnProperty('adapter')) {
      this.adapter = options.adapter;
    } else {
      throw new Error('You must provide a network adapter for the client!');
    }

    this.urlPrefix = options.urlPrefix || '/';
  }

  _createClass(Client, [{
    key: "buildLinkFor",
    value: function buildLinkFor(type) {
      var id = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
      return (0, _url.resolve)(this.urlPrefix, _path["default"].join(type, id, '/'));
    }
  }, {
    key: "request",
    value: function request(_ref) {
      var _ref$method = _ref.method,
          method = _ref$method === void 0 ? 'GET' : _ref$method,
          url = _ref.url,
          data = _ref.data,
          _ref$headers = _ref.headers,
          headers = _ref$headers === void 0 ? {} : _ref$headers;
      var settings = {
        method: method,
        url: url,
        headers: Object.assign(headers, {
          accept: 'application/vnd.api+json',
          'content-type': 'application/vnd.api+json'
        })
      };

      if (data) {
        settings.data = JSON.stringify(data);
      }

      return this.adapter(settings);
    }
  }, {
    key: "fetchRelatedResource",
    value: function fetchRelatedResource(resource, rel) {
      var params = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      var query = params.query;
      var relatedResource = getRelationship(resource, rel);
      var method = 'GET';
      var url = (0, _url.format)({
        pathname: getLink(relatedResource, 'related'),
        query: query
      });
      return this.request({
        method: method,
        url: url
      });
    }
  }, {
    key: "fetchResource",
    value: function fetchResource(resource) {
      var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var query = params.query;
      var type = resource.type,
          id = resource.id;
      var method = 'GET';
      var url = (0, _url.format)({
        pathname: getLink(resource, 'self', this.buildLinkFor(type, id)),
        query: query
      });
      return this.request({
        method: method,
        url: url
      });
    }
  }, {
    key: "findResources",
    value: function findResources(type) {
      var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var query = params.query;
      var method = 'GET';
      var url = (0, _url.format)({
        pathname: this.buildLinkFor(type),
        query: query
      });
      return this.request({
        method: method,
        url: url
      });
    }
  }, {
    key: "updateResource",
    value: function updateResource(resource) {
      var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var query = params.query;
      var type = resource.type,
          id = resource.id;
      var method = 'PATCH';
      var url = (0, _url.format)({
        pathname: getLink(resource, 'self', this.buildLinkFor(type, id)),
        query: query
      });
      var data = {
        data: resource
      };
      return this.request({
        method: method,
        url: url,
        data: data
      });
    }
  }, {
    key: "createResource",
    value: function createResource(resource) {
      var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var query = params.query;
      var type = resource.type;
      var method = 'POST';
      var url = (0, _url.format)({
        pathname: this.buildLinkFor(type),
        query: query
      });
      var data = {
        data: resource
      };
      return this.request({
        method: method,
        url: url,
        data: data
      });
    }
  }]);

  return Client;
}();

exports.Client = Client;

